import 'package:apus_drones_mobile/constants/OrderStatus.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/AddressModel.dart';
import 'package:apus_drones_mobile/models/CreateTripModel.dart';
import 'package:apus_drones_mobile/models/PartnerOrderModel.dart';
import 'package:apus_drones_mobile/models/TripModel.dart';
import 'package:apus_drones_mobile/screens/Appbar.dart';
import 'package:apus_drones_mobile/screens/PilotOrdersWaitingScreen.dart';
import 'package:apus_drones_mobile/services/OrderService.dart';
import 'package:apus_drones_mobile/services/PartnerService.dart';
import 'package:apus_drones_mobile/services/TripService.dart';
import 'package:apus_drones_mobile/widgets/BotNavBar.dart';
import 'package:apus_drones_mobile/widgets/Fee.dart';
import 'package:apus_drones_mobile/widgets/PartnerOrderDetailsHeader.dart';
import 'package:apus_drones_mobile/widgets/ProductCard.dart';
import 'package:apus_drones_mobile/widgets/StatusButton.dart';
import 'package:flutter/material.dart';

class PilotOrderDetailsScreen extends StatefulWidget {
  final int id;
  final String type;
  PilotOrderDetailsScreen({Key? key, required this.id, this.type = 'order'})
      : super(key: key);

  @override
  _PilotOrderDetailsState createState() => _PilotOrderDetailsState();
}

class _PilotOrderDetailsState extends State<PilotOrderDetailsScreen> {
  late PartnerOrderModel order;
  late TripModel trip;
  PartnerService partnerService = new PartnerService();
  OrderService orderService = new OrderService();
  TripService tripService = new TripService();
  bool orderReceived = false;

  _fetchOrder() {
    if (widget.type == 'trip') {
      tripService.getTrip(widget.id).then((o) => setState(() {
            trip = o;
            // Alerta de gambiarra: Não façam isso em casa
            order = PartnerOrderModel(
              id: o.order.id,
              customer: o.order.customer,
              partner: o.order.partner,
              status: o.order.status,
              createdAt: o.order.createdAt,
              expiresAt: DateTime.now().add(Duration(days: 1)),
              items: o.order.items,
              orderPrice: o.order.orderPrice,
              deliveryPrice: o.order.deliveryPrice,
              deliveryAddress: o.order.deliveryAddress ??
                  AddressModel(description: '', number: 0, lat: 0, lng: 0),
              shopAddress: o.order.shopAddress ??
                  AddressModel(description: '', number: 0, lat: 0, lng: 0),
            );
            orderReceived = true;
          }));
    } else {
      partnerService.getOrder(widget.id).then((o) => this.setState(() {
            order = o;
            orderReceived = true;
          }));
    }
  }

  @override
  void initState() {
    super.initState();
    orderReceived = false;
    this._fetchOrder();
  }

  @override
  Widget build(BuildContext context) {
    return render();
  }

  Scaffold render() {
    if (orderReceived) {
      return Scaffold(
          appBar: Appbar(title: 'Pedido ${order.id}'),
          bottomNavigationBar: BotNavBar(role: UserType.PARTNER),
          body: Container(
            margin: EdgeInsets.symmetric(horizontal: 12.0),
            child: Column(
              children: [
                Padding(
                  padding: EdgeInsets.all(16),
                  child: Container(
                    child: StatusButton(status: this.order.status),
                  ),
                ),
                PartnerOrderDetailsHeader(order: this.order),
                Container(
                    alignment: Alignment.centerLeft,
                    margin: EdgeInsets.only(left: 12.0),
                    child: Text(
                      'Produtos:',
                      style: TextStyle(fontSize: 20),
                    )),
                Expanded(child: _generateItems()),
                Container(
                  margin: EdgeInsets.only(
                    top: 25.0,
                  ),
                  child: Column(
                    children: _buildFooter(context),
                  ),
                ),
              ],
            ),
          ));
    }
    return Scaffold();
  }

  ListView _generateItems() {
    List<ProductCard> products = this
        .order
        .items
        .map((item) => ProductCard(
              product: item.product,
              quantity: item.quantity,
            ))
        .toList();
    return ListView.builder(
      shrinkWrap: true,
      itemBuilder: (context, index) => Column(
        children: products,
      ),
      itemCount: products.length,
    );
  }

  List<Widget> _buildFooter(BuildContext context) {
    List<Widget> footernWidgets = [
      Padding(
        padding: const EdgeInsets.only(bottom: 20.0),
        child: Fee(
            productsSumPrice: this.order.orderPrice,
            deliveryFee: this.order.deliveryPrice,
            totalPrice: _totalPrice(this.order)),
      )
    ];

    if (this.order.status == OrderStatus.WAITING_FOR_PILOT) {
      footernWidgets
          .add(_buildButton(context, "Coletar pedido", OrderStatus.PICKING_UP));
    } else if (this.order.status == OrderStatus.PICKING_UP) {
      footernWidgets
          .add(_buildButton(context, "Pedido coletado", OrderStatus.IN_FLIGHT));
    } else if (this.order.status == OrderStatus.IN_FLIGHT) {
      footernWidgets
          .add(_buildButton(context, "Pedido entregue", OrderStatus.DELIVERED));
    }
    return footernWidgets;
  }

  Widget _buildButton(
      BuildContext context, String buttonMessage, OrderStatus toStatus) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 15.0),
      child: Align(
        alignment: Alignment.bottomCenter,
        child: ElevatedButton(
          onPressed: () {
            switch (toStatus) {
              case OrderStatus.PICKING_UP:
                _createTrip().then((_) => Navigator.pop(context));
                break;
              case OrderStatus.IN_FLIGHT:
                tripService
                    .setTripCollected(trip.id)
                    .then((_) => Navigator.pop(context));
                break;
              case OrderStatus.DELIVERED:
                tripService
                    .setTripDelivered(trip.id)
                    .then((_) => Navigator.pop(context));
                break;
              default:
                _updateOrderStatus(toStatus)
                    .then((_) => {Navigator.pop(context)});
            }
          },
          style: ElevatedButton.styleFrom(
              padding:
                  EdgeInsets.only(top: 12, left: 60, right: 60, bottom: 12),
              shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.all(Radius.circular(4)))),
          child: Text(
            buttonMessage,
            style: TextStyle(
              fontSize: 16,
              color: Colors.white,
            ),
          ),
        ),
      ),
    );
  }

  Future<TripModel> _createTrip() async {
    final _trip = CreateTripModel(orderId: order.id);
    return tripService.createTrip(_trip);
  }

  Future<PartnerOrderModel?> _updateOrderStatus(OrderStatus toStatus) {
    this.order.status = toStatus;
    return orderService.updateOrder(this.order);
  }
}

double _totalPrice(PartnerOrderModel order) =>
    order.orderPrice + order.deliveryPrice;
