import 'dart:async';

import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/OrderModel.dart';
import 'package:apus_drones_mobile/models/PartnerOrderModel.dart';
import 'package:apus_drones_mobile/screens/Appbar.dart';
import 'package:apus_drones_mobile/screens/PilotOrderDetailScreen.dart';
import 'package:apus_drones_mobile/services/OrderService.dart';
import 'package:apus_drones_mobile/widgets/BotNavBar.dart';
import 'package:apus_drones_mobile/widgets/Order.dart';
import 'package:apus_drones_mobile/widgets/PartnerOrderModal.dart';
import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:modal_bottom_sheet/modal_bottom_sheet.dart';

class PilotOrdersWaitingScreen extends StatefulWidget {
  static final String routeName = 'pilot/ordersWaiting';
  static final int botNavItemIndex = 3;

  @override
  _PilotOrdersWaitingScreenState createState() =>
      _PilotOrdersWaitingScreenState();
}

class _PilotOrdersWaitingScreenState extends State<PilotOrdersWaitingScreen> {
  List<OrderModel> orderList = [];
  OrderService orderService = new OrderService();
  bool show = false;
  bool modalOpen = false;
  late PartnerOrderModel order;

  @override
  void initState() {
    super.initState();
    orderList = [];
    orderService = new OrderService();
    show = false;
    modalOpen = false;
    this._fetchOrders();
    // startTimer();
  }

  // void startTimer() {
  //   tick(callNewOrders, 30, 1);
  //   tick(showWaitingOrdersModal, 15, 2);
  // }

  void tick(Function func, int startTime, int tickId) {
    int time = startTime;
    const oneSec = const Duration(seconds: 1);
    Timer.periodic(oneSec, (Timer timer) {
      if (!mounted) {
        timer.cancel();
        return;
      }

      if (ModalRoute.of(context)!.isCurrent) {
        this.setState(() {});
        if (time <= 0) {
          time = startTime;
          func();
        }
        time--;
        print('Tick ID $tickId tempo: $time');
      }
    });
  }

  // void callNewOrders() {
  //   this._fetchOrders('WAITING_FOR_PARTNER');
  //   if (show) {
  //     LocalNotification n = LocalNotification();
  //     n.displayNotification(order.customer.name);
  //   }
  // }

  // void showWaitingOrdersModal() {
  //   if (!modalOpen) {
  //     for (int i = 0; i < orderList.length; i++) {
  //       if (orderList[i].status == OrderStatus.WAITING_FOR_PARTNER) {
  //         order = orderList[i];
  //         show = true;
  //         i = orderList.length;
  //       }
  //     }
  //   }
  // }

  _fetchOrders() {
    orderService.getOrdersWaitingForPilot().then((orders) => setState(() {
          orderList = orders;
        }));
  }

  // _changeStatus(OrderStatus status) {
  //   String filter = status.value;
  //   if (status == OrderStatus.ALL) {
  //     filter = '';
  //   }
  //   orderService.getOrdersWaitingForPilot().then((orders) => this.setState(() {
  //         orderList = orders;
  //       }));
  // }

  void changeModalOpenToFalse() {
    modalOpen = false;
  }

  void showModal(BuildContext context) {
    if (show) {
      show = false;
      modalOpen = true;
      SchedulerBinding.instance!.addPostFrameCallback((_) {
        showMaterialModalBottomSheet(
            context: context,
            builder: (context) =>
                PartnerOrderModal(order, refresh, changeModalOpenToFalse));
      });
    }
  }

  void refresh() {
    this.setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    showModal(context);
    return Scaffold(
      appBar: Appbar(
        title: "Pedidos aguardando coleta",
      ),
      bottomNavigationBar: BotNavBar(role: UserType.PILOT),
      body: Center(
        child: Column(
          children: [
            Container(
                constraints: BoxConstraints(maxWidth: 200),
                child: Padding(
                  padding: EdgeInsets.all(16),
                  child: Text(
                    "Os pedidos estão ordenados por data, você verá os pedidos mais recentes primeiro",
                    textAlign: TextAlign.center,
                    style: TextStyle(fontSize: 12, fontStyle: FontStyle.italic),
                  ),
                )),
            // Container(
            //   child: Padding(
            //       padding: EdgeInsets.all(12),
            //       child: FilterOrderStatusMenu(
            //           startingStatus: OrderStatus.ALL,
            //           changeStatusCallback: this._changeStatus)),
            // ),
            Expanded(
              child: ListView.builder(
                itemBuilder: (ctx, index) => Order(
                  model: this.orderList[index],
                  onTap: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) => PilotOrderDetailsScreen(
                            id: this.orderList[index].id),
                      ),
                    ).then((value) {
                      this._fetchOrders();
                    });
                  },
                ),
                itemCount: this.orderList.length,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
