import 'dart:async';
import 'dart:convert';

import 'package:apus_drones_mobile/assets/config.dart';
import 'package:apus_drones_mobile/models/CreateTripModel.dart';
import 'package:apus_drones_mobile/models/TripModel.dart';
import 'package:apus_drones_mobile/utils/httpHeaders.dart';
import 'package:http/http.dart' as http;

class TripService {
  Future<List<TripModel>> getPilotTrips() async {
    http.Response response = await http.get(
      Uri.parse('${Config.apiURL}/trips/by-pilot'),
      headers: await getHttpHeaders(),
    );

    List<dynamic> rsBody = json.decode(response.body);
    List<TripModel> trips =
        rsBody.map((trip) => TripModel.fromJson(trip)).toList();

    return trips;
  }

  Future<TripModel> getTrip(int tripId) async {
    http.Response response = await http.get(
      Uri.parse('${Config.apiURL}/trips/$tripId'),
      headers: await getHttpHeaders(),
    );

    Map<String, dynamic> rsBody = json.decode(response.body);
    TripModel tripCreated = TripModel.fromJson(rsBody);

    return tripCreated;
  }

  Future<TripModel> createTrip(CreateTripModel trip) async {
    http.Response response = await http.post(
      Uri.parse('${Config.apiURL}/trips'),
      headers: await getHttpHeaders(),
      body: json.encode(trip.toJson()),
    );

    Map<String, dynamic> rsBody = json.decode(response.body);
    TripModel tripCreated = TripModel.fromJson(rsBody);

    return tripCreated;
  }

  Future<TripModel> setTripCollected(int tripId) async {
    http.Response response = await http.patch(
      Uri.parse('${Config.apiURL}/trips/$tripId/collected'),
      headers: await getHttpHeaders(),
    );

    Map<String, dynamic> rsBody = json.decode(response.body);
    TripModel tripCreated = TripModel.fromJson(rsBody);

    return tripCreated;
  }

  Future<TripModel> setTripDelivered(int tripId) async {
    http.Response response = await http.patch(
      Uri.parse('${Config.apiURL}/trips/$tripId/delivered'),
      headers: await getHttpHeaders(),
    );

    Map<String, dynamic> rsBody = json.decode(response.body);
    TripModel tripCreated = TripModel.fromJson(rsBody);

    return tripCreated;
  }
}
