import 'package:apus_drones_mobile/constants/Constants.dart';
import 'package:apus_drones_mobile/models/ProductModel.dart';
import 'package:apus_drones_mobile/widgets/Price.dart';
import 'package:flutter/material.dart';

class ProductCard extends StatelessWidget {
  final ProductModel product;
  final bool selected;
  final int quantity;
  final void Function()? onTap;
  final void Function()? onLongPress;
  const ProductCard(
      {Key? key,
      required this.product,
      this.selected = false,
      required this.quantity,
      this.onTap,
      this.onLongPress})
      : super(key: key);

  Color _getCardColor() {
    if (this.selected) {
      return Colors.red.shade50;
    }
    if (this.quantity > 0) {
      return Colors.grey.shade50;
    }
    return Colors.white;
  }

  Color? _getShadowColor() {
    if (this.selected) {
      return Colors.red.shade500;
    }
    return null;
  }

  Color _getTextColor() {
    return this.quantity > 0 ? Colors.black : Colors.grey;
  }

  @override
  Widget build(BuildContext context) {
    return Card(
      color: _getCardColor(),
      shadowColor: _getShadowColor(),
      elevation: 3,
      margin: EdgeInsets.symmetric(horizontal: 14, vertical: 8),
      child: Padding(
        padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 3),
        child: ListTile(
          onTap: this.onTap,
          onLongPress: this.onLongPress,
          title: Text(
            this.product.name,
            style: TextStyle(fontSize: 18, color: _getTextColor()),
            overflow: TextOverflow.ellipsis,
          ),
          leading: Container(
            height: 60,
            width: 60,
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(4),
              image: new DecorationImage(
                  image:
                      new NetworkImage(this.product.imageUrl ?? IMG_NO_PHOTO),
                  fit: BoxFit.cover,
                  alignment: FractionalOffset.center),
            ),
          ),
          trailing: Column(children: [
            Price(
              price: this.product.price,
              disabled: this.quantity == 0,
            ),
            Text('${this.quantity} unid.',
                style: TextStyle(color: _getTextColor())),
          ]),
          subtitle: Container(
            padding: EdgeInsets.only(top: 4.0),
            child:
                Column(crossAxisAlignment: CrossAxisAlignment.start, children: [
              Text(this.product.description,
                  style: TextStyle(color: _getTextColor()),
                  overflow: TextOverflow.ellipsis),
              Text('${this.product.weight} g',
                  style: TextStyle(color: _getTextColor()))
            ]),
          ),
        ),
      ),
    );
  }
}
