import 'package:apus_drones_mobile/models/BasicProfileModel.dart';
import 'package:apus_drones_mobile/providers/AuthProvider.dart';
import 'package:apus_drones_mobile/services/BasicProfileService.dart';
import 'package:flutter/material.dart';

class ProfileMenuItem {
  final IconData icon;
  final String title;
  final void Function() onMenuClicked;

  const ProfileMenuItem(
      {required this.icon, required this.title, required this.onMenuClicked});
}

class ProfileModal extends StatefulWidget {
  final List<ProfileMenuItem> items;
  final int idUser;
  const ProfileModal({Key? key, required this.items, required this.idUser})
      : super(key: key);

  @override
  _ProfileModalState createState() => _ProfileModalState();
}

class _ProfileModalState extends State<ProfileModal> {
  BasicProfileService basicProfileService = new BasicProfileService();

  String name = "";
  String email = "";
  String avatarUrl = "";
  String erro = "";
  bool isLoading = false;

  @override
  void initState() {
    this._getProfileData();
    super.initState();
  }

  void _getProfileData() async {
    this.setState(() {
      this.isLoading = true;
    });
    try {
      BasicProfileModel profile =
          await basicProfileService.getBasicProfile(this.widget.idUser);

      this.setState(() {
        this.name = profile.name;
        this.email = profile.email;
        this.avatarUrl = profile.avatarUrl;
        this.isLoading = false;
      });
    } catch (err) {
      setState(() {
        this.erro = "Falha ao buscar dados";
        this.isLoading = false;
      });
    }
  }

  Text _getTitle() {
    return this.erro.isEmpty
        ? Text(
            this.name,
            style: TextStyle(fontSize: 20),
          )
        : Text(
            this.erro,
            style: TextStyle(
              fontSize: 20,
              fontStyle: FontStyle.italic,
              color: Colors.grey,
            ),
          );
  }

  @override
  Widget build(BuildContext context) {
    return this.isLoading
        ? Center(
            child: CircularProgressIndicator(),
          )
        : Container(
            child: Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                SizedBox(
                    height: 280,
                    width: double.infinity,
                    child: Container(
                        child: Padding(
                      padding: const EdgeInsets.all(15.0),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        mainAxisSize: MainAxisSize.max,
                        children: [
                          CircleAvatar(
                            radius: 30.0,
                            backgroundImage: NetworkImage(this.avatarUrl),
                            backgroundColor: this.avatarUrl.isEmpty
                                ? Colors.grey
                                : Colors.transparent,
                          ),
                          SizedBox(
                            height: 10,
                          ),
                          Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              this._getTitle(),
                              Text(
                                this.email,
                                style: TextStyle(fontSize: 20),
                              )
                            ],
                          ),
                          Padding(
                            padding: const EdgeInsets.only(top: 10),
                            child: Divider(),
                          ),
                          ...this
                              .widget
                              .items
                              .map((item) => this.renderItem(item))
                              .toList(),
                        ],
                      ),
                    ))),
              ],
            ),
          );
  }

  Widget renderItem(ProfileMenuItem item) {
    return GestureDetector(
      onTap: () => item.onMenuClicked(),
      child: Padding(
        padding: const EdgeInsets.symmetric(vertical: 10),
        child: Row(
          children: [
            Icon(
              item.icon,
              size: 30,
              color: Colors.black54,
            ),
            SizedBox(
              width: 10,
            ),
            Text(
              item.title,
              style: TextStyle(fontSize: 16),
            ),
          ],
        ),
      ),
    );
  }
}
